package cn.com.qmth.examcloud.task.service.job;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.stereotype.Component;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.commons.util.PathUtil;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCourseReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCourseResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveStudentResp;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.task.dao.ExamStudentImportRepo;
import cn.com.qmth.examcloud.task.dao.ExamStudentTempRepo;
import cn.com.qmth.examcloud.task.dao.entity.ExamStudentImportEntity;
import cn.com.qmth.examcloud.task.dao.entity.ExamStudentTempEntity;
import cn.com.qmth.examcloud.task.dao.enums.ExamStudentImportStatus;
import cn.com.qmth.examcloud.web.config.SystemProperties;
import cn.com.qmth.examcloud.web.task.AbstractTask;
import cn.com.qmth.examcloud.web.task.ScheduleJob;
import cn.com.qmth.examcloud.web.task.TaskTracker;

/**
 * 考生导入-文件解析
 *
 * @author WANGWEI
 * @date 2018年7月31日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
@Component("examStudentImportDataProcessingTask")
public class ExamStudentImportDataProcessingTask extends AbstractTask {

	@Autowired
	SystemProperties systemConfig;

	private static final String EXAM_STUDENT_IMPORT_FILES = "exam_student_import_files";

	@Autowired
	TaskTracker TaskTracker;

	@Autowired
	ExamStudentImportRepo examStudentImportRepo;

	@Autowired
	ExamStudentTempRepo examStudentTempRepo;

	@Autowired
	CourseCloudService courseCloudService;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamCloudService examCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@Override
	public void run(ScheduleJob scheduleJob) throws Exception {

		ExamStudentImportEntity importEntity = examStudentImportRepo
				.findFirstByStatusOrderByCreationTime(ExamStudentImportStatus.DATA_PROCESSING);

		if (null == importEntity) {
			importEntity = examStudentImportRepo.findFirstByStatusOrderByCreationTime(
					ExamStudentImportStatus.FILE_PARSING_COMPLETE);
		}

		if (null == importEntity) {
			return;
		}

		importEntity.setStatus(ExamStudentImportStatus.DATA_PROCESSING);
		examStudentImportRepo.saveAndFlush(importEntity);

		process(importEntity);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param importEntity
	 */
	private void process(ExamStudentImportEntity importEntity) {
		List<Map<String, Object>> failRecords = Collections
				.synchronizedList(new ArrayList<Map<String, Object>>());

		foreach(importEntity, failRecords);

		StringBuilder sb = new StringBuilder();
		for (Map<String, Object> cur : failRecords) {
			sb.append("line ").append(cur.get("lineNum")).append(cur.get("msg")).append("\n");
		}
		sb.append("====================异常数据\n");
		String resultFilePath = PathUtil.getCanonicalPath(systemConfig.getDataDir() + "/"
				+ EXAM_STUDENT_IMPORT_FILES + "/" + importEntity.getResultFilePath());
		File resultFile = new File(resultFilePath);

		try {
			FileUtils.writeStringToFile(resultFile, sb.toString(), "UTF-8", true);
		} catch (IOException e) {
			debugLog.error("导入报告写入失败", e);
			importEntity.setStatus(ExamStudentImportStatus.ERROR);
			importEntity.setErrorDesc("导入报告写入失败");
			examStudentImportRepo.saveAndFlush(importEntity);
			return;
		}

		importEntity.setStatus(ExamStudentImportStatus.DATA_PROCESSING_COMPLETE);
		examStudentImportRepo.saveAndFlush(importEntity);

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param importEntity
	 * @param failRecords
	 */
	private void foreach(ExamStudentImportEntity importEntity,
			List<Map<String, Object>> failRecords) {
		Long batchId = importEntity.getBatchId();

		while (true) {
			Pageable pageable = PageRequest.of(0, 100, Sort.Direction.ASC, "lineNum");
			List<ExamStudentTempEntity> list = examStudentTempRepo
					.findByBatchIdAndStatusCodeIsNull(batchId, pageable);
			if (CollectionUtils.isEmpty(list)) {
				break;
			}

			for (ExamStudentTempEntity cur : list) {
				saveExamStudent(cur, failRecords);
			}
		}

	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param entity
	 * @param failRecords
	 */
	private void saveExamStudent(ExamStudentTempEntity entity,
			List<Map<String, Object>> failRecords) {
		Long rootOrgId = entity.getRootOrgId();
		String courseCode = entity.getCourseCode();
		String orgCode = entity.getOrgCode();

		int error = 0;
		int disabledCount = 0;
		Map<String, Object> map = Maps.newHashMap();
		map.put("lineNum", entity.getLineNum());

		try {
			GetCourseReq req = new GetCourseReq();
			req.setRootOrgId(rootOrgId);
			req.setCode(courseCode);
			GetCourseResp resp = courseCloudService.getCourse(req);
			CourseBean courseBean = resp.getCourseBean();
			entity.setCourseId(courseBean.getId());

			if (null != courseBean.getEnable() && (!courseBean.getEnable())) {
				map.put("msg", "  " + "课程被禁用");
				entity.setStatusCode("T-890101");
				entity.setStatusDesc("课程被禁用");
				disabledCount++;
				error++;
			}

		} catch (StatusException e) {
			debugLog.error("查询课程异常", e);
			map.put("msg", "  " + e.getDesc());
			entity.setStatusCode(e.getCode());
			entity.setStatusDesc(e.getDesc());
			error++;
		}

		try {
			GetOrgReq req = new GetOrgReq();
			req.setOrgCode(orgCode);
			req.setRootOrgId(rootOrgId);
			GetOrgResp resp = orgCloudService.getOrg(req);
			OrgBean org = resp.getOrg();
			entity.setOrgId(org.getId());

			if (null != org.getEnable() && (!org.getEnable())) {
				map.put("msg", "  " + "机构被禁用");
				entity.setStatusCode("T-890100");
				entity.setStatusDesc("机构被禁用");
				disabledCount++;
				error++;
			}

		} catch (StatusException e) {
			debugLog.error("查询机构异常", e);
			map.put("msg", "  " + e.getDesc());
			entity.setStatusCode(e.getCode());
			entity.setStatusDesc(e.getDesc());
			error++;
		}

		if (disabledCount == 2) {
			map.put("msg", "  " + "机构和课程被禁用");
			entity.setStatusCode("T-890105");
			entity.setStatusDesc("机构和课程被禁用");
		}

		if (error > 0) {
			failRecords.add(map);
			examStudentTempRepo.saveAndFlush(entity);
			return;
		}

		try {
			SaveStudentReq saveStudentReq = new SaveStudentReq();
			saveStudentReq.setIdentityNumber(entity.getIdentityNumber());
			saveStudentReq.setName(entity.getName());
			saveStudentReq.setPhoneNumber(entity.getPhone());
			saveStudentReq.setOrgCode(entity.getOrgCode());
			saveStudentReq.setOrgId(entity.getOrgId());
			saveStudentReq.setRootOrgId(rootOrgId);
			if (StringUtils.isNotBlank(entity.getStudentCode())) {
				List<String> studentCodeList = Lists.newArrayList();
				studentCodeList.add(entity.getStudentCode());
				saveStudentReq.setStudentCodeList(studentCodeList);
			}

			SaveStudentResp saveStudentResp = studentCloudService.saveStudent(saveStudentReq);

			Long studentId = saveStudentResp.getStudentId();

			SaveExamStudentReq sReq = new SaveExamStudentReq();

			sReq.setCourseId(entity.getCourseId());
			sReq.setCourseCode(entity.getCourseCode());

			sReq.setExamId(entity.getExamId());
			sReq.setRootOrgId(rootOrgId);

			sReq.setStudentId(studentId);
			sReq.setIdentityNumber(entity.getIdentityNumber());
			sReq.setStudentName(entity.getName());
			sReq.setStudentCode(entity.getStudentCode());

			sReq.setSpecialtyName(entity.getSpecialtyName());
			sReq.setPaperType(entity.getPaperType());
			sReq.setGrade(entity.getGrade());
			sReq.setInfoCollector(entity.getInfoCollector());
			sReq.setExamSite(entity.getExamSite());

			sReq.setSpecialBeginTime(entity.getSpecialBeginTime());
			sReq.setSpecialEndTime(entity.getSpecialEndTime());

			sReq.setExt1(entity.getExt1());
			sReq.setExt2(entity.getExt2());
			sReq.setExt3(entity.getExt3());
			sReq.setExt4(entity.getExt4());
			sReq.setExt5(entity.getExt5());

			examStudentCloudService.saveExamStudent(sReq);

			entity.setStudentId(studentId);
			entity.setStatusCode("200");
			entity.setStatusDesc("成功");
			examStudentTempRepo.saveAndFlush(entity);

		} catch (StatusException e) {
			debugLog.error("考生入库异常", e);
			map.put("msg", "  " + e.getDesc());
			failRecords.add(map);
			entity.setStatusCode(e.getCode());
			entity.setStatusDesc(e.getDesc());
			examStudentTempRepo.saveAndFlush(entity);
			return;
		} catch (Exception e) {
			// ignore
			taskLog.error("unexpected.", e);
		}

	}

	@Override
	public TaskTracker getTaskTracker() {
		return TaskTracker;
	}
}
